/**
 * @file tv_text_rs.h
 * TV_Text native device driver interface
 */
#define cls 0x00
#define home 0x02     
#define SPACE 0x20
#define BS 0x08
#define tab 0x09
#define esc 0x01
#define RETN 0x0d
#define up 0xc1
#define down 0xc2
#define right 0xc3
#define left 0xc4
#define buflen 8000
 
/**
 * TV_Text color indicies
 */
#define TV_TEXT_WHITE_BLUE     0
#define TV_TEXT_WHITE_RED      1
#define TV_TEXT_YELLOW_BROWN   2
#define TV_TEXT_GREY_WHITE     3
#define TV_TEXT_CYAN_DARKCYAN  4
#define TV_TEXT_GREEN_WHITE    5
#define TV_TEST_RED_PINK       6
#define TV_TEXT_CYAN_BLUE      7

#define TV_TEXT_COLORS 8

/**
 * TV_Text palette color indicies
 */
#define TV_TEXT_PAL_WHITE_BLUE     0
#define TV_TEXT_PAL_WHITE_RED      2
#define TV_TEXT_PAL_YELLOW_BROWN   4
#define TV_TEXT_PAL_GREY_WHITE     6
#define TV_TEXT_PAL_CYAN_DARKCYAN  8
#define TV_TEXT_PAL_GREEN_WHITE    10
#define TV_TEST_PAL_RED_PINK       12
#define TV_TEXT_PAL_CYAN_BLUE      14


/**
 * TV_Text color table size.
 * Table holds foreground and background info, so size is 2 x table colors.
 */
#define TV_TEXT_COLORTABLE_SIZE 8*2
#define lastrow TV_TEXT_SCREENSIZE - TV_TEXT_COLS
/**
 * TV_Text column count
 */
#define  TV_TEXT_COLS 40

/**
 * TV_Text row count
 */
#define  TV_TEXT_ROWS 12

/**
 * TV_Text screensize count
 */
#define  TV_TEXT_SCREENSIZE (TV_TEXT_COLS * TV_TEXT_ROWS)

/**
 * TV_Text lastrow position count
 */
#define  TV_TEXT_LASTROW (TV_TEXT_SCREENSIZE-TV_TEXT_COLS)

/**Keyboard 
*/
#define ASIO_TX_PIN_MAP (1 << 30)

#define ASIO_RX_PIN_MAP (1 << 31)
/**
 * Use large number (100+) of TX stop bits for slow interfaces adjust at your liesure
 * Flow control not implemented - use 3 wire interface (rx,tx,gnd).
 */
#define ASIO_STOP_BITS  2    // 1 - N
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_9600   56   
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_19200  22   
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_38400  7 
/**
 * Define for RX baud rate using software + waitcount delays.
 */ 
#define ASIO_BAUDX_RX_57600  2  
/* Use these defines for software + waitcount RX delays.
 * The use of waitcnt instruction asm("waitcnt %time, %period")
 * dominates timing at lower baudrates; software delays are
 * a factor in higher baudrates, but are mostly mitigated.
 */ 
#define ASIO_BAUDX_RX   ASIO_BAUDX_RX_57600    

/* Base period is used with ASIO_BAUDX_dX for bit period calculations.
 */
#define ASIO_BASE_PERIOD    120
/**
 * Use this define to invert the interface sense (or not)
 */
#define ASIO_RT_INVERTED    0
/**
 * Use this define to invert the RX interface sense (or not)
 */
#define ASIO_RX_INVERTED    ASIO_RT_INVERTED

#define MAP(x)  { x, ASIO_BAUDX_RX_##x }   
/*
 * TV_Text status enum
 */
typedef enum {
    TV_TEXT_STAT_DISABLED,
    TV_TEXT_STAT_INVISIBLE,
    TV_TEXT_STAT_VISIBLE
} tvTextStat_t;

/**
 * TV_Text control struct
 */
typedef struct _tv_text_struct
{
    long status     ; //0/1/2 = off/invisible/visible              read-only
    long enable     ; //0/non-0 = off/on                           write-only
    long pins       ; //%pppmmmm = pin group, pin group mode       write-only
    long mode       ; //%tccip = tile,chroma,interlace,ntsc/pal    write-only
    long screen     ; //pointer to screen (words)                  write-only      
    long colors     ; //pointer to colors (longs)                  write-only                            
    long ht         ; //horizontal tiles                           write-only                            
    long vt         ; //vertical tiles                             write-only                            
    long hx         ; //horizontal tile expansion                  write-only                            
    long vx         ; //vertical tile expansion                    write-only                            
    long ho         ; //horizontal offset                          write-only                            
    long vo         ; //vertical offset                            write-only                            
    long broadcast  ; //broadcast frequency (Hz)                   write-only                            
    long auralcog   ; //aural fm cog                               write-only      
} tvText_t;


 /* TV_Text start function starts TV on a cog
 * @param basepin is first pin number (out of 8) connected to TV
 * param clockrate is the clockrate defined for the platform.
 * @returns non-zero cogid on success
 */
 
int     tvText_start(int basepin);


/**
 * TV_Text str function prints a string at current position
 * @param sptr is string to print
 */
void    tvText_str(char* sptr);


void    tvText_out(int c);


